@@ -46,8 +46,6 @@ typedef enum {
 #include "version.h"
 
 #ifdef _MSC_VER
-#define strncasecmp _strnicmp
-#define strcasecmp _stricmp
 #define strdup _strdup
 #endif
 
@@ -74,6 +72,11 @@ typedef enum {
 #if CURL_AT_LEAST_VERSION(7,30,0)
 #define SUPPORTS_IMAP_OPTIONS
 #endif
+#if CURL_AT_LEAST_VERSION(8,9,0)
+#define SUPPORTS_NO_GUESS_SCHEME
+#else
+#define CURLU_NO_GUESS_SCHEME 0
+#endif
 
 #define OUTPUT_URL      0  /* default */
 #define OUTPUT_SCHEME   1
@@ -151,6 +154,52 @@ static char *curl_url_strerror(CURLUcode error)
 }
 #endif
 
+/* Mapping table to go from lowercase to uppercase for plain ASCII.*/
+static const unsigned char touppermap[256] = {
+0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21,
+22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40,
+41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59,
+60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78,
+79, 80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90, 91, 92, 93, 94, 95, 96, 65,
+66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82, 83, 84,
+85, 86, 87, 88, 89, 90, 123, 124, 125, 126, 127, 128, 129, 130, 131, 132, 133,
+134, 135, 136, 137, 138, 139, 140, 141, 142, 143, 144, 145, 146, 147, 148, 149,
+150, 151, 152, 153, 154, 155, 156, 157, 158, 159, 160, 161, 162, 163, 164, 165,
+166, 167, 168, 169, 170, 171, 172, 173, 174, 175, 176, 177, 178, 179, 180, 181,
+182, 183, 184, 185, 186, 187, 188, 189, 190, 191, 192, 193, 194, 195, 196, 197,
+198, 199, 200, 201, 202, 203, 204, 205, 206, 207, 208, 209, 210, 211, 212, 213,
+214, 215, 216, 217, 218, 219, 220, 221, 222, 223, 224, 225, 226, 227, 228, 229,
+230, 231, 232, 233, 234, 235, 236, 237, 238, 239, 240, 241, 242, 243, 244, 245,
+246, 247, 248, 249, 250, 251, 252, 253, 254, 255
+};
+
+/* Portable, ASCII-consistent toupper. Do not use toupper() because its
+   behavior is altered by the current locale. */
+#define raw_toupper(in) touppermap[(unsigned int)in]
+
+/*
+ * casecompare() does ASCII based case insensitive checks, as a strncasecmp
+ * replacement.
+ */
+
+static int casecompare(const char *first, const char *second, size_t max)
+{
+  while(*first && *second && max) {
+    int diff = raw_toupper(*first) - raw_toupper(*second);
+    if(diff)
+      /* get out of the loop as soon as they don't match */
+      return diff;
+    max--;
+    first++;
+    second++;
+  }
+  if(!max)
+    return 0; /* identical to this point */
+
+  return raw_toupper(*first) - raw_toupper(*second);
+}
+#define strncasecmp(x,y,z) casecompare(x,y,z)
+
 static void message_low(const char *prefix, const char *suffix,
                         const char *fmt, va_list ap)
 {
@@ -232,30 +281,33 @@ static void show_version(void)
   fprintf(stdout, "%s version %s libcurl/%s [built-with %s]\n",
           PROGNAME, TRURL_VERSION_TXT, data->version, LIBCURL_VERSION);
   fprintf(stdout, "features:");
+#ifdef SUPPORTS_IMAP_OPTIONS
+  if(supports_imap)
+    fprintf(stdout, " imap-options");
+#endif
+#ifdef SUPPORTS_NO_GUESS_SCHEME
+  fprintf(stdout, " no-guess-scheme");
+#endif
+#ifdef SUPPORTS_NORM_IPV4
+  fprintf(stdout, " normalize-ipv4");
+#endif
 #ifdef SUPPORTS_PUNYCODE
   if(supports_puny)
     fprintf(stdout, " punycode");
 #endif
+#ifdef SUPPORTS_PUNY2IDN
+  if(supports_puny)
+    fprintf(stdout, " punycode2idn");
+#endif
+#ifdef SUPPORTS_URL_STRERROR
+  fprintf(stdout, " url-strerror");
+#endif
 #ifdef SUPPORTS_ALLOW_SPACE
   fprintf(stdout, " white-space");
 #endif
 #ifdef SUPPORTS_ZONEID
   fprintf(stdout, " zone-id");
 #endif
-#ifdef SUPPORTS_URL_STRERROR
-  fprintf(stdout, " url-strerror");
-#endif
-#ifdef SUPPORTS_NORM_IPV4
-  fprintf(stdout, " normalize-ipv4");
-#endif
-#ifdef SUPPORTS_IMAP_OPTIONS
-  if(supports_imap)
-    fprintf(stdout, " imap-options");
-#endif
-#ifdef SUPPORTS_PUNY2IDN
-  if(supports_puny)
-    fprintf(stdout, " punycode2idn");
-#endif
 
   fprintf(stdout, "\n");
   exit(0);
@@ -757,6 +809,7 @@ static void get(struct option *o, CURLU *uh)
         size_t vlen;
         bool isquery = false;
         bool queryall = false;
+        bool strict = false; /* strict mode, fail on URL decode problems */
         int mods = 0;
         end = strchr(ptr, endbyte);
         ptr++; /* pass the { */
@@ -766,39 +819,44 @@ static void get(struct option *o, CURLU *uh)
           continue;
         }
 
-        /* {path} {:path} */
+        /* {path} {:path} {/path} */
         if(*ptr == ':') {
           mods |= VARMODIFIER_URLENCODED;
           ptr++;
         }
         vlen = end - ptr;
         do {
+          size_t wordlen;
           cl = memchr(ptr, ':', vlen);
           if(!cl)
             break;
+          wordlen = cl - ptr + 1;
 
           /* modifiers! */
-          if(!strncmp(ptr, "default:", cl - ptr + 1))
+          if(!strncmp(ptr, "default:", wordlen))
             mods |= VARMODIFIER_DEFAULT;
-          else if(!strncmp(ptr, "puny:", cl - ptr + 1)) {
+          else if(!strncmp(ptr, "puny:", wordlen)) {
             if(mods & VARMODIFIER_PUNY2IDN)
               errorf(o, ERROR_GET,
                      "puny modifier is mutually exclusive with idn");
             mods |= VARMODIFIER_PUNY;
           }
-          else if(!strncmp(ptr, "idn:", cl - ptr + 1)) {
+          else if(!strncmp(ptr, "idn:", wordlen)) {
             if(mods & VARMODIFIER_PUNY)
               errorf(o, ERROR_GET,
                      "idn modifier is mutually exclusive with puny");
             mods |= VARMODIFIER_PUNY2IDN;
           }
+          else if(!strncmp(ptr, "strict:", wordlen))
+            strict = true;
+          else if(!strncmp(ptr, "url:", wordlen))
+            mods |= VARMODIFIER_URLENCODED;
           else {
-            /* {query: or {query-all: */
-            if(!strncmp(ptr, "query-all:", cl - ptr + 1)) {
+            if(!strncmp(ptr, "query-all:", wordlen)) {
               isquery = true;
               queryall = true;
             }
-            else if(!strncmp(ptr, "query:", cl - ptr + 1))
+            else if(!strncmp(ptr, "query:", wordlen))
               isquery = true;
             else {
               /* syntax error */
@@ -844,7 +902,10 @@ static void get(struct option *o, CURLU *uh)
               /* silently ignore */
               break;
             default:
-              trurl_warnf(o, "%s (%s)\n", curl_url_strerror(rc), v->name);
+              if((rc == CURLUE_URLDECODE) && strict)
+                errorf(o, ERROR_GET, "problems URL decoding %s", v->name);
+              else
+                trurl_warnf(o, "%s (%s)", curl_url_strerror(rc), v->name);
               break;
             }
           }
@@ -929,7 +990,7 @@ static const struct var *setone(CURLU *uh, const char *setline,
 
       if(conditional) {
         char *piece;
-        rc = curl_url_get(uh, v->part, &piece, 0);
+        rc = curl_url_get(uh, v->part, &piece, CURLU_NO_GUESS_SCHEME);
         if(!rc) {
           skip = true;
           curl_free(piece);
